Package: varbvs
Encoding: UTF-8
Type: Package
Version: 2.6-10
Date: 2023-05-31
Title: Large-Scale Bayesian Variable Selection Using Variational
        Methods
Authors@R: c(person("Peter","Carbonetto",role=c("aut","cre"),
                    email="peter.carbonetto@gmail.com"),
             person("Matthew","Stephens",role="aut"),
             person("David","Gerard",role="ctb"))
Maintainer: Peter Carbonetto <peter.carbonetto@gmail.com>
Description: Fast algorithms for fitting Bayesian variable selection
    models and computing Bayes factors, in which the outcome (or
    response variable) is modeled using a linear regression or a
    logistic regression. The algorithms are based on the variational
    approximations described in "Scalable variational inference for
    Bayesian variable selection in regression, and its accuracy in
    genetic association studies" (P. Carbonetto & M. Stephens, 2012,
    <DOI:10.1214/12-BA703>). This software has been applied to large
    data sets with over a million variables and thousands of samples.
Depends: R (>= 3.1.0)
Imports: methods, Matrix, stats, graphics, lattice, latticeExtra, Rcpp,
        nor1mix
Suggests: curl, glmnet, qtl, knitr, rmarkdown, testthat
License: GPL (>= 3)
NeedsCompilation: yes
LazyData: true
URL: https://github.com/pcarbo/varbvs
BugReports: https://github.com/pcarbo/varbvs/issues
LinkingTo: Rcpp
VignetteBuilder: knitr
Packaged: 2023-05-31 17:20:08 UTC; pcarbo
Author: Peter Carbonetto [aut, cre],
  Matthew Stephens [aut],
  David Gerard [ctb]
Repository: CRAN
Date/Publication: 2023-05-31 18:30:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 01:29:41 UTC; unix
Archs: varbvs.so.dSYM
